@@ -1,5 +1,6 @@
 extern crate chrono;
 
+use chrono::DateTime;
 use chrono::Datelike;
 use chrono::NaiveDate;
 use chrono::NaiveDateTime;
@@ -19,20 +20,17 @@ pub fn get_unix_timestamp_ms() -> i64 {
 }
 
 pub fn time_stamp_to_date(time_stamp: i64) -> String {
-    let utc = NaiveDateTime::from_timestamp_opt(time_stamp, 0).unwrap();
-    let datetime = Utc.from_utc_datetime(&utc);
+    let datetime = DateTime::from_timestamp(time_stamp, 0).unwrap();
     datetime.format(CHRONO_DATE_FORMAT).to_string()
 }
 
 pub fn time_stamp_to_time(time_stamp: i64) -> String {
-    let utc = NaiveDateTime::from_timestamp_opt(time_stamp, 0).unwrap();
-    let datetime = Utc.from_utc_datetime(&utc);
+    let datetime = DateTime::from_timestamp(time_stamp, 0).unwrap();
     datetime.format(CHRONO_TIME_FORMAT).to_string()
 }
 
 pub fn time_stamp_to_date_time(time_stamp: i64) -> String {
-    let utc = NaiveDateTime::from_timestamp_opt(time_stamp, 0).unwrap();
-    let datetime = Utc.from_utc_datetime(&utc);
+    let datetime = DateTime::from_timestamp(time_stamp, 0).unwrap();
     datetime.format(CHRONO_DATE_TIME_FULL_FORMAT).to_string()
 }
 
@@ -40,7 +38,7 @@ pub fn date_to_time_stamp(date: &str) -> i64 {
     let date_time = NaiveDate::parse_from_str(date, CHRONO_DATE_FORMAT).ok();
     if let Some(date) = date_time {
         let zero_time = NaiveTime::from_hms_opt(0, 0, 0).unwrap();
-        return date.and_time(zero_time).timestamp();
+        return date.and_time(zero_time).and_utc().timestamp();
     }
     0
 }
@@ -56,39 +54,39 @@ pub fn date_time_to_time_stamp(date: &str) -> i64 {
     if date_time.is_err() {
         return 0;
     }
-    date_time.ok().unwrap().timestamp()
+    date_time.ok().unwrap().and_utc().timestamp()
 }
 
 pub fn date_time_to_hour(date: i64) -> i64 {
-    let date_time = NaiveDateTime::from_timestamp_opt(date, 0);
+    let date_time = DateTime::from_timestamp(date, 0);
     let dt = date_time.unwrap().time();
     dt.hour() as i64
 }
 
 pub fn date_time_to_minute(date: i64) -> i64 {
-    let date_time = NaiveDateTime::from_timestamp_opt(date, 0);
+    let date_time = DateTime::from_timestamp(date, 0);
     let dt = date_time.unwrap().time();
     dt.minute() as i64
 }
 
 pub fn date_to_day_number_in_week(date: i64) -> u32 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
     parsed_date.weekday().number_from_sunday()
 }
 
 pub fn date_to_day_number_in_month(date: i64) -> u32 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
     parsed_date.day()
 }
 
 pub fn date_to_day_number_in_year(date: i64) -> u32 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
     parsed_date.ordinal()
 }
 
 pub fn date_to_week_number_in_year(date: i64) -> u32 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
-    let native_date = parsed_date.date();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
+    let native_date = parsed_date.date_naive();
     let first_day_of_year = NaiveDate::from_ymd_opt(native_date.year(), 1, 1).unwrap();
     let days_diff = native_date
         .signed_duration_since(first_day_of_year)
@@ -103,7 +101,7 @@ pub fn date_to_week_number_in_year(date: i64) -> u32 {
 }
 
 pub fn date_to_day_name(date: i64) -> String {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
 
     let day_name = match parsed_date.weekday() {
         Weekday::Mon => "Monday",
@@ -119,7 +117,7 @@ pub fn date_to_day_name(date: i64) -> String {
 }
 
 pub fn date_to_month_name(date: i64) -> String {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
 
     let month_name = match parsed_date.month() {
         1 => "January",
@@ -141,28 +139,28 @@ pub fn date_to_month_name(date: i64) -> String {
 }
 
 pub fn date_to_quarter_index(date: i64) -> i64 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
     let month = parsed_date.month() as i64;
     (month - 1) / 3 + 1
 }
 
 pub fn date_to_year(date: i64) -> i32 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
     parsed_date.year()
 }
 
 pub fn date_to_month(date: i64) -> u32 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
     parsed_date.month()
 }
 
 pub fn date_to_weekday(date: i64) -> u32 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
     parsed_date.weekday().number_from_monday() - 1
 }
 
 pub fn date_to_days_count(date: i64) -> i64 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
     let days_since_year_0 = parsed_date.ordinal0() as i64;
     let year = parsed_date.year() as i64;
     let leap_years = (year - 1) / 4 - (year - 1) / 100 + (year - 1) / 400;
@@ -171,7 +169,7 @@ pub fn date_to_days_count(date: i64) -> i64 {
 }
 
 pub fn date_last_day(date: i64) -> i64 {
-    let parsed_date = NaiveDateTime::from_timestamp_opt(date, 0).unwrap();
+    let parsed_date = DateTime::from_timestamp(date, 0).unwrap();
     let (year, month) = (parsed_date.year(), parsed_date.month());
 
     let curr_month_start = NaiveDate::from_ymd_opt(year, month, 1).unwrap();
