@@ -1,5 +1,6 @@
 use crate::date_utils as GitQLDate;
 use crate::regex_utils as GitQLRegex;
+use crate::signature::Signature;
 use crate::types::same_type_as_first_parameter;
 use crate::types::DataType;
 use crate::value::Value;
@@ -12,14 +13,10 @@ use rand::SeedableRng;
 use std::cmp::Ordering;
 use std::collections::HashMap;
 use std::ops::Rem;
+use uuid::Uuid;
 
 type Function = fn(&[Value]) -> Value;
 
-pub struct Prototype {
-    pub parameters: Vec<DataType>,
-    pub result: DataType,
-}
-
 lazy_static! {
     pub static ref FUNCTIONS: HashMap<&'static str, Function> = {
         let mut map: HashMap<&'static str, Function> = HashMap::new();
@@ -50,6 +47,7 @@ lazy_static! {
         map.insert("unicode", text_unicode);
         map.insert("strcmp", text_strcmp);
         map.insert("quotename", text_quotename);
+        map.insert("str", text_str);
 
         // Date functions
         map.insert("date", date_extract_date);
@@ -94,12 +92,13 @@ lazy_static! {
         map.insert("mod", numeric_mod);
         map.insert("rand", numeric_rand);
 
-        // Other Functions
+        // General Functions
         map.insert("isnull", general_is_null);
         map.insert("isnumeric", general_is_numeric);
         map.insert("typeof", general_type_of);
         map.insert("greatest", general_greatest);
         map.insert("least", general_least);
+        map.insert("uuid", general_uuid);
 
         // Regex Functions
         map.insert("regexp_instr", regexp_instr);
@@ -111,525 +110,543 @@ lazy_static! {
 }
 
 lazy_static! {
-    pub static ref PROTOTYPES: HashMap<&'static str, Prototype> = {
-        let mut map: HashMap<&'static str, Prototype> = HashMap::new();
+    pub static ref PROTOTYPES: HashMap<&'static str, Signature> = {
+        let mut map: HashMap<&'static str, Signature> = HashMap::new();
         // String functions
         map.insert(
             "lower",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "upper",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "reverse",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "replicate",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Integer],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "space",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Integer],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "trim",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "ltrim",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "rtrim",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "len",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             },
         );
         map.insert(
             "ascii",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             },
         );
         map.insert(
             "left",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Integer],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "datalength",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             },
         );
         map.insert(
             "char",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Integer],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "nchar",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Integer],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "charindex",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Text],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "replace",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Text, DataType::Text],
-                result: DataType::Text
+                return_type: DataType::Text
           },
         );
         map.insert(
             "substring",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Integer, DataType::Integer],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "stuff",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Integer, DataType::Integer, DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "right",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Integer],
-                result: DataType::Text
+                return_type: DataType::Text
              },
         );
         map.insert(
             "translate",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Text, DataType::Text],
-                result: DataType::Text
+                return_type: DataType::Text
              },
         );
         map.insert(
             "soundex",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "concat",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Any, DataType::Any, DataType::Varargs(Box::new(DataType::Any))],
-                result: DataType::Text
+                return_type: DataType::Text
              },
         );
         map.insert(
             "concat_ws",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Any, DataType::Any, DataType::Varargs(Box::new(DataType::Any))],
-                result: DataType::Text
+                return_type: DataType::Text
              },
         );
         map.insert(
             "unicode",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text],
-                result: DataType::Integer
+                return_type: DataType::Integer
              },
         );
-        map.insert("strcmp", Prototype { parameters: vec![DataType::Text, DataType::Text], result: DataType::Integer });
+        map.insert("strcmp", Signature { parameters: vec![DataType::Text, DataType::Text], return_type: DataType::Integer });
         map.insert(
             "quotename",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Optional(Box::new(DataType::Text))],
-                result: DataType::Text
+                return_type: DataType::Text
+            }
+        );
+        map.insert(
+            "str",
+            Signature {
+                parameters: vec![
+                    DataType::Variant(vec![DataType::Integer, DataType::Float]),
+                    DataType::Optional(Box::new(DataType::Integer)),
+                    DataType::Optional(Box::new(DataType::Integer))
+                ],
+                return_type: DataType::Text
             }
         );
 
         // Date functions
         map.insert(
             "date",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Variant(vec![DataType::Date, DataType::DateTime])],
-                result: DataType::Date,
+                return_type: DataType::Date,
             },
         );
         map.insert(
             "current_date",
-            Prototype {
+            Signature {
                 parameters: vec![],
-                result: DataType::Date,
+                return_type: DataType::Date,
             },
         );
         map.insert(
             "current_time",
-            Prototype {
+            Signature {
                 parameters: vec![],
-                result: DataType::Time,
+                return_type: DataType::Time,
             },
         );
         map.insert(
             "current_timestamp",
-            Prototype {
+            Signature {
                 parameters: vec![],
-                result: DataType::DateTime,
+                return_type: DataType::DateTime,
             },
         );
         map.insert(
             "now",
-            Prototype {
+            Signature {
                 parameters: vec![],
-                result: DataType::DateTime,
+                return_type: DataType::DateTime,
             },
         );
         map.insert(
             "makedate",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Integer, DataType::Integer],
-                result: DataType::Date,
+                return_type: DataType::Date,
             },
         );
         map.insert(
             "maketime",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Integer, DataType::Integer, DataType::Integer],
-                result: DataType::Time,
+                return_type: DataType::Time,
             },
         );
         map.insert(
             "dayname",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Text,
+                return_type: DataType::Text,
             }
         );
         map.insert(
             "day",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "monthname",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Text,
+                return_type: DataType::Text,
             }
         );
         map.insert(
             "hour",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::DateTime],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "minute",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::DateTime],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "isdate",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Any],
-                result: DataType::Boolean,
+                return_type: DataType::Boolean,
             }
         );
         map.insert(
             "dayofweek",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "dayofmonth",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "dayofyear",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "weekofyear",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "quarter",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "year",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "month",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "weekday",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "to_days",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             }
         );
         map.insert(
             "last_day",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Date,
+                return_type: DataType::Date,
             }
         );
         map.insert(
             "yearweek",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Date],
-                result: DataType::Text,
+                return_type: DataType::Text,
             }
         );
         // Numeric functions
         map.insert(
             "abs",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Variant(vec![DataType::Integer, DataType::Float])],
-                result: DataType::Dynamic(same_type_as_first_parameter),
+                return_type: DataType::Dynamic(same_type_as_first_parameter),
             },
         );
         map.insert(
             "pi",
-            Prototype {
+            Signature {
                 parameters: vec![],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         map.insert(
             "floor",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Float],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             },
         );
         map.insert(
             "round",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Float, DataType::Optional(Box::new(DataType::Integer))],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         map.insert(
             "square",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Integer],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             },
         );
         map.insert(
             "sin",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Float],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         map.insert(
             "asin",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Float],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         map.insert(
             "cos",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Float],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         map.insert(
             "acos",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Float],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         map.insert(
             "tan",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Float],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         map.insert(
             "atan",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Float],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         map.insert(
             "atn2",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Float, DataType::Float],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         map.insert(
             "sign",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Variant(vec![DataType::Integer, DataType::Float])],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             },
         );
         map.insert(
             "mod",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Integer, DataType::Integer],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             },
         );
         map.insert(
             "rand",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Optional(Box::new(DataType::Float))],
-                result: DataType::Float,
+                return_type: DataType::Float,
             },
         );
         // General functions
         map.insert(
             "isnull",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Any],
-                result: DataType::Boolean,
+                return_type: DataType::Boolean,
             },
         );
         map.insert(
             "isnumeric",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Any],
-                result: DataType::Boolean,
+                return_type: DataType::Boolean,
             },
         );
         map.insert(
             "typeof",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Any],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "greatest",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Any, DataType::Any, DataType::Varargs(Box::new(DataType::Any))],
-                result: DataType::Any
+                return_type: DataType::Any
              },
         );
         map.insert(
             "least",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Any, DataType::Any, DataType::Varargs(Box::new(DataType::Any))],
-                result: DataType::Any
+                return_type: DataType::Any
              },
         );
+        map.insert(
+            "uuid",
+            Signature {
+                parameters: vec![],
+                return_type: DataType::Text,
+            },
+        );
 
         // Regex functions
         map.insert(
             "regexp_instr",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Text],
-                result: DataType::Integer,
+                return_type: DataType::Integer,
             },
         );
         map.insert(
             "regexp_like",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Text],
-                result: DataType::Boolean,
+                return_type: DataType::Boolean,
             },
         );
         map.insert(
             "regexp_replace",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Text, DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map.insert(
             "regexp_substr",
-            Prototype {
+            Signature {
                 parameters: vec![DataType::Text, DataType::Text],
-                result: DataType::Text,
+                return_type: DataType::Text,
             },
         );
         map
@@ -749,8 +766,8 @@ fn text_replace(inputs: &[Value]) -> Value {
 
 fn text_substring(inputs: &[Value]) -> Value {
     let text = inputs[0].as_text();
-    //according to the specs, a stirng starts at position 1.
-    //but in Rust, the index of a string starts from 0
+    // According to the specs, a string starts at position 1.
+    // but in Rust, the index of a string starts from 0
     let start = inputs[1].as_int() as usize - 1;
     let length = inputs[2].as_int();
 
@@ -761,7 +778,10 @@ fn text_substring(inputs: &[Value]) -> Value {
         return Value::Text("".to_string());
     }
 
-    Value::Text(text[start..(start + length as usize)].to_string())
+    // Convert it to Vec<Char> to be easy to substring with support of unicode
+    let chars: Vec<char> = text.chars().collect();
+    let slice = &chars[start..(start + length as usize)];
+    Value::Text(slice.iter().collect())
 }
 
 fn text_stuff(inputs: &[Value]) -> Value {
@@ -897,6 +917,46 @@ fn text_quotename(inputs: &[Value]) -> Value {
     }
 }
 
+fn text_str(inputs: &[Value]) -> Value {
+    let value = &inputs[0];
+    let length = if inputs.len() == 3 {
+        inputs[1].as_int()
+    } else {
+        10
+    };
+
+    let decimals = if inputs.len() == 3 {
+        inputs[2].as_int()
+    } else {
+        0
+    };
+
+    if value.data_type().is_int() {
+        let int_value = value.as_int();
+        let number_string = format!("{:.dec$}", int_value, dec = decimals as usize);
+        if length > 0 {
+            if (length as usize) < number_string.len() {
+                return Value::Text(number_string[..length as usize].to_owned());
+            } else {
+                return Value::Text(format!("{:<len$}", number_string, len = length as usize));
+            }
+        }
+        return Value::Text(number_string.clone());
+    }
+
+    let float_value = value.as_float();
+    let number_string = format!("{:.dec$}", float_value, dec = decimals as usize);
+    if length > 0 {
+        if (length as usize) < number_string.len() {
+            return Value::Text(number_string[..length as usize].to_owned());
+        } else {
+            return Value::Text(format!("{:<len$}", number_string, len = length as usize));
+        }
+    }
+
+    Value::Text(number_string.clone())
+}
+
 // Date functions
 fn date_extract_date(inputs: &[Value]) -> Value {
     let argument_type = inputs[0].data_type();
@@ -1175,6 +1235,11 @@ fn general_least(inputs: &[Value]) -> Value {
     least.to_owned()
 }
 
+fn general_uuid(_inputs: &[Value]) -> Value {
+    let uuid = Uuid::new_v4();
+    Value::Text(uuid.to_string())
+}
+
 // Regex functions
 
 fn regexp_instr(inputs: &[Value]) -> Value {
