@@ -12,7 +12,8 @@ use crate::tokenizer::Token;
 use crate::tokenizer::TokenKind;
 use crate::type_checker::are_types_equals;
 use crate::type_checker::check_all_values_are_same_type;
-use crate::type_checker::is_expression_type_equals;
+use crate::type_checker::check_function_call_arguments;
+use crate::type_checker::type_check_selected_fields;
 use crate::type_checker::TypeCheckResult;
 
 use gitql_ast::aggregation::AGGREGATIONS;
@@ -515,7 +516,12 @@ fn parse_select_statement(
     }
 
     // Type check all selected fields has type registered in type table
-    type_check_selected_fields(env, table_name, &fields_names, tokens, *position)?;
+    type_check_selected_fields(
+        env,
+        table_name,
+        &fields_names,
+        get_safe_location(tokens, *position),
+    )?;
 
     Ok(Box::new(SelectStatement {
         table_name: table_name.to_string(),
@@ -1719,7 +1725,7 @@ fn parse_function_call_expression(
             let mut arguments = parse_arguments_expressions(context, env, tokens, position)?;
             let prototype = PROTOTYPES.get(function_name.as_str()).unwrap();
             let parameters = &prototype.parameters;
-            let mut return_type = prototype.result.clone();
+            let mut return_type = prototype.return_type.clone();
             if let DataType::Dynamic(calculate_type) = return_type {
                 return_type = calculate_type(parameters);
             }
@@ -1746,8 +1752,8 @@ fn parse_function_call_expression(
         if AGGREGATIONS.contains_key(function_name.as_str()) {
             let mut arguments = parse_arguments_expressions(context, env, tokens, position)?;
             let prototype = AGGREGATIONS_PROTOS.get(function_name.as_str()).unwrap();
-            let parameters = &[prototype.parameter.clone()];
-            let mut return_type = prototype.result.clone();
+            let parameters = &prototype.parameters.clone();
+            let mut return_type = prototype.return_type.clone();
             if let DataType::Dynamic(calculate_type) = return_type {
                 return_type = calculate_type(parameters);
             }
@@ -2059,166 +2065,6 @@ fn parse_case_expression(
     }))
 }
 
-fn check_function_call_arguments(
-    env: &mut Environment,
-    arguments: &mut [Box<dyn Expression>],
-    parameters: &[DataType],
-    function_name: String,
-    location: Location,
-) -> Result<(), Box<Diagnostic>> {
-    let parameters_len = parameters.len();
-    let arguments_len = arguments.len();
-
-    let mut has_optional_parameter = false;
-    let mut has_varargs_parameter = false;
-    if !parameters.is_empty() {
-        let last_parameter = parameters.last().unwrap();
-        has_optional_parameter = last_parameter.is_optional();
-        has_varargs_parameter = last_parameter.is_varargs();
-    }
-
-    // Has Optional parameter type at the end
-    if has_optional_parameter {
-        // If function last parameter is optional make sure it at least has
-        if arguments_len < parameters_len - 1 {
-            return Err(Box::new(
-                Diagnostic::error(&format!(
-                    "Function `{}` expects at least `{}` arguments but got `{}`",
-                    function_name,
-                    parameters_len - 1,
-                    arguments_len
-                ))
-                .with_location(location),
-            ));
-        }
-
-        // Make sure function with optional parameter not called with too much arguments
-        if arguments_len > parameters_len {
-            return Err(Box::new(
-                Diagnostic::error(&format!(
-                    "Function `{}` expects at most `{}` arguments but got `{}`",
-                    function_name, parameters_len, arguments_len
-                ))
-                .with_location(location),
-            ));
-        }
-    }
-    // Has Variable arguments parameter type at the end
-    else if has_varargs_parameter {
-        // If function last parameter is optional make sure it at least has
-        if arguments_len < parameters_len - 1 {
-            return Err(Box::new(
-                Diagnostic::error(&format!(
-                    "Function `{}` expects at least `{}` arguments but got `{}`",
-                    function_name,
-                    parameters_len - 1,
-                    arguments_len
-                ))
-                .with_location(location),
-            ));
-        }
-    }
-    // No Optional or Variable arguments but has invalid number of arguments passed
-    else if arguments_len != parameters_len {
-        return Err(Box::new(
-            Diagnostic::error(&format!(
-                "Function `{}` expects `{}` arguments but got `{}`",
-                function_name, parameters_len, arguments_len
-            ))
-            .with_location(location),
-        ));
-    }
-
-    let mut last_required_parameter_index = parameters_len;
-    if has_optional_parameter || has_varargs_parameter {
-        last_required_parameter_index -= 1;
-    }
-
-    // Check each argument vs parameter type
-    for index in 0..last_required_parameter_index {
-        let parameter_type = parameters.get(index).unwrap();
-        let argument = arguments.get(index).unwrap();
-        match is_expression_type_equals(env, argument, parameter_type) {
-            TypeCheckResult::Equals => {}
-            TypeCheckResult::RightSideCasted(new_expr) => {
-                arguments[index] = new_expr;
-            }
-            TypeCheckResult::LeftSideCasted(new_expr) => {
-                arguments[index] = new_expr;
-            }
-            TypeCheckResult::NotEqualAndCantImplicitCast => {
-                let argument_type = argument.expr_type(env);
-                return Err(Diagnostic::error(&format!(
-                    "Function `{}` argument number {} with type `{}` don't match expected type `{}`",
-                    function_name, index, argument_type, parameter_type
-                ))
-                .with_location(location).as_boxed());
-            }
-            TypeCheckResult::Error(error) => return Err(error),
-        }
-    }
-
-    // Check the optional or varargs parameters if exists
-    if has_optional_parameter || has_varargs_parameter {
-        let last_parameter_type = parameters.get(last_required_parameter_index).unwrap();
-
-        for index in last_required_parameter_index..arguments_len {
-            let argument = arguments.get(index).unwrap();
-            match is_expression_type_equals(env, argument, last_parameter_type) {
-                TypeCheckResult::Equals => {}
-                TypeCheckResult::RightSideCasted(new_expr) => {
-                    arguments[index] = new_expr;
-                }
-                TypeCheckResult::LeftSideCasted(new_expr) => {
-                    arguments[index] = new_expr;
-                }
-                TypeCheckResult::NotEqualAndCantImplicitCast => {
-                    let argument_type = arguments.get(index).unwrap().expr_type(env);
-                    if !last_parameter_type.eq(&argument_type) {
-                        return Err(Diagnostic::error(&format!(
-                            "Function `{}` argument number {} with type `{}` don't match expected type `{}`",
-                            function_name, index, argument_type, last_parameter_type
-                        ))
-                        .with_location(location).as_boxed());
-                    }
-                }
-                TypeCheckResult::Error(error) => return Err(error),
-            }
-        }
-    }
-
-    Ok(())
-}
-
-fn type_check_selected_fields(
-    env: &mut Environment,
-    table_name: &str,
-    fields_names: &Vec<String>,
-    tokens: &[Token],
-    position: usize,
-) -> Result<(), Box<Diagnostic>> {
-    for field_name in fields_names {
-        if let Some(data_type) = env.resolve_type(field_name) {
-            if data_type.is_undefined() {
-                return Err(Box::new(
-                    Diagnostic::error(&format!("No field with name `{}`", field_name))
-                        .with_location(get_safe_location(tokens, position)),
-                ));
-            }
-            continue;
-        }
-
-        return Err(Diagnostic::error(&format!(
-            "Table `{}` has no field with name `{}`",
-            table_name, field_name
-        ))
-        .add_help("Check the documentations to see available fields for each tables")
-        .with_location(get_safe_location(tokens, position))
-        .as_boxed());
-    }
-    Ok(())
-}
-
 fn un_expected_statement_error(tokens: &[Token], position: &mut usize) -> Box<Diagnostic> {
     let token: &Token = &tokens[*position];
     let location = token.location;
