@@ -171,9 +171,19 @@ pub fn tokenize(script: String) -> Result<Vec<Token>, Box<Diagnostic>> {
             continue;
         }
 
-        // String literal
+        // String literal between single quotes '...'
+        if char == '\'' {
+            tokens.push(consume_string_in_single_quotes(
+                &characters,
+                &mut position,
+                &mut column_start,
+            )?);
+            continue;
+        }
+
+        // String literal between double quotes "..."
         if char == '"' {
-            tokens.push(consume_string(
+            tokens.push(consume_string_in_double_quotes(
                 &characters,
                 &mut position,
                 &mut column_start,
@@ -649,9 +659,7 @@ fn consume_global_variable_name(
 
     // Identifier is being case-insensitive by default, convert to lowercase to be easy to compare and lookup
     let literal = &chars[*start..*pos];
-    let string = String::from_utf8(literal.iter().map(|&c| c as u8).collect())
-        .unwrap()
-        .to_lowercase();
+    let string: String = literal.iter().collect();
 
     let location = Location {
         start: *start,
@@ -661,7 +669,7 @@ fn consume_global_variable_name(
     Ok(Token {
         location,
         kind: TokenKind::GlobalVariable,
-        literal: string,
+        literal: string.to_lowercase(),
     })
 }
 
@@ -672,9 +680,8 @@ fn consume_identifier(chars: &[char], pos: &mut usize, start: &mut usize) -> Tok
 
     // Identifier is being case-insensitive by default, convert to lowercase to be easy to compare and lookup
     let literal = &chars[*start..*pos];
-    let string = String::from_utf8(literal.iter().map(|&c| c as u8).collect())
-        .unwrap()
-        .to_lowercase();
+    let mut string: String = literal.iter().collect();
+    string = string.to_lowercase();
 
     let location = Location {
         start: *start,
@@ -709,7 +716,7 @@ fn consume_number(
     }
 
     let literal = &chars[*start..*pos];
-    let string = String::from_utf8(literal.iter().map(|&c| c as u8).collect()).unwrap();
+    let string: String = literal.iter().collect();
     let literal_num = string.replace('_', "");
 
     let location = Location {
@@ -745,7 +752,7 @@ fn consume_backticks_identifier(
     *pos += 1;
 
     let literal = &chars[*start + 1..*pos - 1];
-    let identifier = String::from_utf8(literal.iter().map(|&c| c as u8).collect()).unwrap();
+    let identifier: String = literal.iter().collect();
 
     let location = Location {
         start: *start,
@@ -783,7 +790,7 @@ fn consume_binary_number(
     }
 
     let literal = &chars[*start..*pos];
-    let string = String::from_utf8(literal.iter().map(|&c| c as u8).collect()).unwrap();
+    let string: String = literal.iter().collect();
     let literal_num = string.replace('_', "");
     let convert_result = i64::from_str_radix(&literal_num, 2);
 
@@ -827,7 +834,7 @@ fn consume_octal_number(
     }
 
     let literal = &chars[*start..*pos];
-    let string = String::from_utf8(literal.iter().map(|&c| c as u8).collect()).unwrap();
+    let string: String = literal.iter().collect();
     let literal_num = string.replace('_', "");
     let convert_result = i64::from_str_radix(&literal_num, 8);
 
@@ -871,7 +878,7 @@ fn consume_hex_number(
     }
 
     let literal = &chars[*start..*pos];
-    let string = String::from_utf8(literal.iter().map(|&c| c as u8).collect()).unwrap();
+    let string: String = literal.iter().collect();
     let literal_num = string.replace('_', "");
     let convert_result = i64::from_str_radix(&literal_num, 16);
 
@@ -893,7 +900,44 @@ fn consume_hex_number(
     })
 }
 
-fn consume_string(
+fn consume_string_in_single_quotes(
+    chars: &[char],
+    pos: &mut usize,
+    start: &mut usize,
+) -> Result<Token, Box<Diagnostic>> {
+    *pos += 1;
+
+    while *pos < chars.len() && chars[*pos] != '\'' {
+        *pos += 1;
+    }
+
+    if *pos >= chars.len() {
+        return Err(Diagnostic::error("Unterminated single quote string")
+            .add_help("Add \' at the end of the String literal")
+            .with_location_span(*start, *pos)
+            .as_boxed());
+    }
+
+    *pos += 1;
+
+    let literal = &chars[*start + 1..*pos - 1];
+    let string: String = literal.iter().collect();
+
+    let location = Location {
+        start: *start,
+        end: *pos,
+    };
+
+    let string_literal = Token {
+        location,
+        kind: TokenKind::String,
+        literal: string,
+    };
+
+    Ok(string_literal)
+}
+
+fn consume_string_in_double_quotes(
     chars: &[char],
     pos: &mut usize,
     start: &mut usize,
@@ -914,7 +958,7 @@ fn consume_string(
     *pos += 1;
 
     let literal = &chars[*start + 1..*pos - 1];
-    let string = String::from_utf8(literal.iter().map(|&c| c as u8).collect()).unwrap();
+    let string: String = literal.iter().collect();
 
     let location = Location {
         start: *start,
